c----------------------------------------------------------------------
      subroutine uamake_comp
c----------------------------------------------------------------------
c
c --- UAMAKE    Version: 1.53            Level: 090411      UAMAKE_COMP
c               K. Morrison, D. Strimaitis
c
c --- PURPOSE:  Process one or more M3D files to produce an
c               UP.DAT file for the specified station location
c
c --- UPDATES:
c
c --- v 1.52 Level 090225  ====> v 1.53 Level 090411 (DGS)
c     1)  Remove standard 12-hour reporting time constraint by
c         adding MTIMES and HRMASK variables
c         Modified: CONTROL.UAS
c                   BLOCK DATA, READCF, SETUP, COMP
c     2)  Use explicit snapshot representation of times, but keep
c         output dataset version 2.1 (all times interpreted as snapshot)
c         Modified: COMP
c
c --- v 1.51 Level 080919  ====> v 1.52 Level 090225 (DGS)
c     1)  Update CALUTILS from V1.0 L010713 to V2.57 L090225
c     2)  Update argument list in call to XTRACTLL
c     3)  Change filename variables from c*70 to c*132
c         Modified: FILNAM.UAS
c                   READCF, SETUP, COMP
c     4)  Add spatial processing method (MSPACE) and code station
c         location for any map projection, but set CF inputs to LL
c         as before (future update may allow others)
c         New:      MAP.UAS, COORDLIB(included)
c                   INTERPH, INTERPHWIND
c         Modified: CONTROL.UAS
c                   BLOCK DATA, READCF, COMP
c     5)  Trap/correct longitude differences that exceed 180 degrees
c         when locating nearest cell.
c         Modified: COMP
c     6)  Add extraction location to output UP.DAT header
c         Modified: COMP, WRHD
c     7)  Previous UP.DAT file used to start run is a formatted file,
c         but was opened unformatted.
c         Modified: SETUP
c     8)  Add ESTRNG for error message reporting, and transfer control
c         back to main program to treat errors (no STOP in subroutines)
c         /ERR/ processing
c         New:      ERR.CAL
c         Modified: (main), BLOCK DATA, SETUP, READCF,
c                   COMP, GETVER, WRHD
c
c --- v 1.5 Level 050208  ====> v 1.51 Level 080919 (DGS)
c     1)  Move the time zone record so that it appears just
c         before the time information record when station
c         location information is written
c         Modified: WRHD
c
c --- v 1.32 Level 050208  ====> v 1.5 Level 050208 (I. Lee)
c     1)  v1.3(work) of UAMAKE was created concurrently with this version,
c         therefore this version is being renamed to v1.5
c
c --- v 1.31 Level 041216  ====> v 1.32 Level 050208 (I. Lee)
c     1)  Added parameter "MXLEV" to "params.uas" for the maximum number
c         of vertical levels allowed. Previously, it had been hard-coded
c         at 40.
c     2)  Added subroutine GETVER (taken from WDEXTRA 1.10,Level:041111)
c         to "uamake_comp.for" which identifies the type/version of input 
c         3D.DAT file. Allows UAMAKE to run on MM4.DAT, MM5.DAT, M3D 
c         format 3D.DAT files.
c
c --- v 1.3 Level 041122  ====> v 1.31 Level 041216 (I. Lee)
c     1)  Subroutine COMP and Subroutine WRHD moved here from UAMAKE.for
c         in order to separate out the main computational code of UAMAKE.
c         There will now be two sets of FORTRAN code that will call to
c         "uamake_comp.for", one will be "uamake_fortran.for" for
c         creating an internal FORTRAN executable, and the other will be
c         "uamake_dll.for" which will be used to create a VISUAL BASIC
c         DLL for use in a distributed GUI.
c         Now, if UAMAKE needs to be updated or modified, this can 
c         be done by just modifying the code once in "uamake_comp.for".
c
c --- V 1.2 Level 041116  ====> V 1.3 Level 041122  (I. Lee)
c           - New output format (2.1) with explicit beginning/ending
c             times (includes seconds)
c           - Update WRHD arguments (new header)

c --- INPUTS:
c
c --- OUTPUT:
c           none
c
c --- COMP called by:  MAIN
c --- COMP calls:      JULDAY, GETVER, WRHD, GLOBE1, GLOBE
c----------------------------------------------------------------------
c --- Include file of parameters and commons
      include 'params.uas'
      include 'control.uas'
      include 'filnam.uas'
      include 'map.uas'
      include 'qa.uas'
      include 'err.cal'

c --- Profile arrays: (5,_) holds current values read in
c --- while (1,_)-(4,_) store nearest 4 points
      real t(5,mxlev),spd(5,mxlev)
      real pres(5,mxlev),ht(5,mxlev),dir(5,mxlev)
      real wgts(4)

      character*1 ccomma
      character*132 buff1,buff2
      character*132 comment(2)
      character*40 line40
      character*128 aline
      logical lfirst, L1,L2,L3,L4

c --- Declare variables from previous UP.DAT header
      character*4 pname
      character*16 plat,plon
      character*16 ptype
      character*12 pver,plev,pmodel

c --- M3D header
      character*80 title
      character*64 datamod
      character*12 cset3d,cver,clev
      character*4 cmap4

c --- For coordinate transformations
      character*8 cmapi,cmapo
      character*12 caction
      character*4 c4hem
      real*8 vecti(9),vecto(9)

c --- Set fixed format data for output
      data ccomma/','/
      data L1/.FALSE./,L2/.FALSE./,L3/.FALSE./,L4/.FALSE./
      data jdat/2/, iform/2/

c --- Announce status
      write(iomesg,*)'COMPUTATIONAL PHASE'

c --- Check for first date/time
      if(LPREV) then
         lfirst=.FALSE.
c ---    Read header of previous output file (UP.DAT format)
         read(ioprev,*) ptype,pmodel,pver,plev
         read(ioprev,*) iyrs,ijuls,ihrs,iyre,ijule,ihre,iptop
         read(ioprev,*) idstap,pname,plat,plon,ielevm
c ---    Report to list file
         write(iolst,90) pmodel,pver,plev,idstap,pname,plat,plon,
     &                   ielevm
90       format(1x,'Data Read from Previous UP.DAT File:'//,
     2         3x,'Processor:  ',a12,/,3x,'Version:  ',a12,/,3x,
     3         'Level:  ',a12,/,3x,'Station ID:',i10,/,3x,'Name  :',
     4         a6,/,3x,'Latitude:  ',a16,/,3x,'Longitude:  ',a16,/,
     5         3x,'Elevation:  ',i5)
         write(iolst,100) ijuls,iyrs,ihrs,ijule,iyre,ihre
100      format(/,3x,'Period:   ',i3,'/',i4,2x,i2,':00','  to  ',
     1          i3,'/',i4,2x,i2,':00')
      else
         lfirst=.TRUE.
         ielevm=100000
      endif


c --- Loop over M3D files
c -----------------------
      do im3d=1,nm3d
         open(ioinp,file=datafil(im3d),status='old')

c ---    Report file
         kjust=LEN_TRIM(justname(im3d))
         write(iomesg,'(a)') justname(im3d)(1:kjust)

c ---    Determine the 3D.DAT file format/version
c ---    Read in first two lines
         read(ioinp,'(a)',end=99)buff1
         read(ioinp,'(a)',end=99)buff2
         call GETVER(buff1,buff2,ivs3)
         if(iret.NE.0) then
            klen=LEN_TRIM(estrng)
            estrng='COMP:'//estrng(1:klen)
            return
         endif

         if(ivs3.eq.0) then       ! V2 (MM5.DAT) Format
c ---       #1 from buffer
            read(buff1,'(a80,2a12)',end=99)title,cver,clev 
c ---       #2 from buffer
            read(buff2,'(6i3)')ioutw,ioutq,ioutc,iouti,ioutg
c ---       #3. Projection parameters
            read(ioinp,'(9x,f6.2,7x,f8.2,8x,f6.1,8x,f6.1)')
     &                   rnlat03d,relon03d,rnlat13d,rnlat23d
            cmap4='LCC '  
c ---       #4. MPHYSICS
            read(ioinp,'(30i3)')inhyd,imphys,icupa,ibltyp,ifrad,isoil,
     &                        ifddaan,ifddaob
c ---       #5
            read(ioinp,'(4i2,i5,3i4)')iyr,imo,idy,ihr,nhrs,nx,ny,nl
c ---       #6
            read(ioinp,'(4i4,4(f8.2))')nx1,ny1,nx2,ny2,
     &                                 rxmin,rxmax,rymin,rymax

         elseif(ivs3.gt.0) then  ! V3 formats
            if(ivs3.eq.1) then   ! (no comment lines)
c ---          #1, #2 from buffer
               read(buff1,'(a80)',end=99)title
               read(buff2,'(3a12)')cset3d,cver,clev
            elseif(ivs3.eq.2) then
c ---          #1 from buffer
               read(buff1,'(2(a12,4x),a64)',end=99)cset3d,cver,datamod
c ---          #2 from buffer
               read(buff2,*)ncomm
               do i=1,ncomm
                  read(ioinp,*)
               enddo
            endif
c ---       #3
            read(ioinp,'(6i3)')ioutw,ioutq,ioutc,iouti,ioutg,iosrf
c ---       #4
            read(ioinp,'(a4,f9.4,f10.4,2f7.2,2f10.3,f8.3,2i4,i3)')
     &                   cmap4,rnlat03d,relon03d,rnlat13d,rnlat23d,
     &                   xsw,ysw,dxm,nx,ny,nz
            dym=dxm
c ---       #5 - physical options
            read(ioinp,'(30i3)')inhyd,imphys,icupa,ibltyp,ifrad,isoil,
     &           ifddaan,ifddaob,igrdt,ipbl,ishf,ilhf,iustr,iswdn,
     &           ilwdn,ist1,ist2,ist3,ist4,ist5,ist6
	
c ---       #6 - Extracted domain Stamp 1
            read(ioinp,'(i4,3i2,i5,3i4)')iyr,imo,idy,ihr,nhrs,nx,ny,nl
c ---       #7: - Extracted domain Stamp 2
            read(ioinp,'(6i4,2f10.4,2f9.4)')nx1,ny1,nx2,ny2,nz1,nz2,
     &                                      rxmin,rxmax,rymin,rymax
         endif

c ---    Process date record
         call JULDAY(iolst,iyr,imo,idy,ijul)
         if(iret.NE.0) then
            klen=LEN_TRIM(estrng)
            estrng='COMP:'//estrng(1:klen)
            return
         endif

c ---    Condition map projection
         if(cmap4.eq.'LCC ' .or. cmap4.eq.'lcc ' .or.
     &      cmap4.eq.'LLC ' .or. cmap4.eq.'llc ' .or.
     &      cmap4.eq.'LC  ') then
            cmap3d='LCC     '
         else
            cmap3d(1:4)=cmap4
            cmap3d(5:8)='    '
            if(cmap3d.EQ.'TTM     ') cmap3d='TM      '
         endif
c ---    Set other map variables
         datum3d='NWS-84  '
c ---    Set unused map variables
         feast3d=0.0
         fnorth3d=0.0

c ---    Coordinate transformation to 3D grid x,y
c ---    Scale factor for Tangential TM projection
         tmsone=1.00000
c ---    Input side
         cmapi=cmap
         iutmi=iutmzn
         if(utmhem.EQ.'S   ' .AND. iutmzn.LT.900) iutmi=-iutmi
c ---    Output side
         cmapo=cmap3d
         iutmo=iutmzn3d
         if(utmhem3d.EQ.'S   ' .AND. iutmzn3d.LT.900) iutmo=-iutmo
c ---    Transform
         call GLOBE1(cmapi,iutmi,tmsone,rnlat1,rnlat2,rnlat0,relon0,
     &        feast,fnorth,
     &        cmapo,iutmo,tmsone,rnlat13d,rnlat23d,rnlat03d,relon03d,
     &        feast3d,fnorth3d,
     &        caction,vecti,vecto)

c ---    Skip level documentation records
         do i=1,nl
            read(ioinp,'(a40)') line40
         enddo

c ---    Select grid point nearest station location in lat/lon space
c ---    as grid points are read
c ---    Note:  M3D uses N latitude and E longitude
         dist=1.e20
         indxx=0
         indxy=0
         do i=1,ny
            do j=1,nx
               read(ioinp,60) ixp,iyp,xlatp,xlonp
60             format(2i4,1x,f8.4,1x,f9.4)
c ---          Trap 0:360 or 180:-180 longitude cut
               absdlon=ABS(relonX-xlonp)
               if(absdlon.GT.180.) absdlon=360.-absdlon
               dista=sqrt((rnlatY-xlatp)**2+(absdlon)**2)
               if(dista<dist) then
                  indxx=ixp
                  indxy=iyp
                  dist=dista
                  xxlat=xlatp
                  xxlon=xlonp
               endif
            enddo
         enddo

c ---    Set nearest grid point location in M3D map projection (x,y)
         xn=xsw+(indxx-1)*dxm
         yn=ysw+(indxy-1)*dxm

c ---    Set station location in M3D map projection(x,y)
         xin=relonX
         yin=rnlatY
         call GLOBE(ilog,caction,datum,vecti,datum3d,vecto,
     &              xin,yin,xs,ys,idum,c4hem)
         if(iret.NE.0) then
            klen=LEN_TRIM(estrng)
            estrng='COMP:'//estrng(1:klen)
            return
         endif
c ---    Set station location as grid(xi,yj) (real, not integer)
         xsi=1.0+(xs-xsw)/dxm
         ysj=1.0+(ys-ysw)/dxm
c ---    Demand that position be within extraction domain +/- half cell
         ijbad=0
         if(NINT(xsi).LT.nx1 .OR. NINT(xsi).GT.nx2) ijbad=1
         if(NINT(ysj).LT.ny1 .OR. NINT(ysj).GT.ny2) ijbad=1
         if(ijbad.GT.0) then
            write(iolst,*)'COMP:  Station location outside M3D grid'
            write(iolst,*)'       Easting  Grid Cells = ',nx1,nx2
            write(iolst,*)'       Northing Grid Cells = ',ny1,ny2
            write(iolst,*)'     Station Location Cell = ',xsi,ysj
            write(iolst,*)'       Nearest Model (I,J) = ',indxx,indxy
c            stop
            iret=1
            estrng='Error in COMP -- see list file'
            return
         endif
c ---    Adjust as needed to place location within domain
         x1=FLOAT(nx1)
         x2=FLOAT(nx2)
         xsi=MAX(xsi,x1)
         xsi=MIN(xsi,x2)
         y1=FLOAT(ny1)
         y2=FLOAT(ny2)
         ysj=MAX(ysj,y1)
         ysj=MIN(ysj,y2)

c ---    Lower-left index of bounding points
         ix1=INT(xsi)
         ix1=MAX(nx1,ix1)
         ix1=MIN(nx2-1,ix1)
         iy1=INT(ysj)
         iy1=MAX(ny1,iy1)
         iy1=MIN(ny2-1,iy1)

c ---    Upper-right index of bounding points
         ix2=ix1+1
         iy2=iy1+1

c ---    Compute interpolation weights
c ---    Full weight at ix1
         wgts(1)=1.-(xsi-FLOAT(ix1))
c ---    Full weight at ix2
         wgts(2)=xsi-FLOAT(ix1)
c ---    Full weight at iy1
         wgts(3)=1.-(ysj-FLOAT(iy1))
c ---    Full weight at iy2
         wgts(4)=ysj-FLOAT(iy1)

c ---    Report weight products actually used for each corner
c ---    (These could have been used directly but retain WGTS array)
         w1=wgts(1)*wgts(3)
         w2=wgts(2)*wgts(3)
         w3=wgts(1)*wgts(4)
         w4=wgts(2)*wgts(4)

c ---    Report file specs to list file
         write(iolst,*)
         write(iolst,*)
         write(iolst,*)
         write(iolst,*)'Processing M3D file: ',justname(im3d)(1:kjust)
         write(iolst,*)'--------------------'
         write(iolst,*)
         write(iolst,*)'Starting Year, Day, Hour : ',iyr,ijul,ihr
         write(iolst,*)'Number of hours in file  : ',nhrs
         write(iolst,*)'Station Extraction Location ---'
         write(iolst,*)'           N-Lat,E-Long  : ',rnlatY,relonX
         write(iolst,*)' Computed Model (X,Y)km  : ',xs,ys
         write(iolst,*)' Computed Model (I,J)    : ',xsi,ysj
         write(iolst,*)'Grid point nearest station ---'
         write(iolst,*)'           N-Lat,E-Long  : ',xxlat,xxlon
         write(iolst,*)'          Model (X,Y)km  : ',xn,yn
         write(iolst,*)'          Model (I,J)    : ',indxx,indxy
         write(iolst,*)'Surrounding grid point weighting ---'
         write(iolst,*)'      Surrounding Cell 1 = ',ix1,iy1,w1
         write(iolst,*)'      Surrounding Cell 2 = ',ix2,iy1,w2
         write(iolst,*)'      Surrounding Cell 3 = ',ix1,iy2,w3
         write(iolst,*)'      Surrounding Cell 4 = ',ix2,iy2,w4

c ---    Demand that nearest point is 1 of 4 surrounding points
         ijnear=0
         if(indxx.EQ.ix1 .AND. indxy.EQ.iy1) ijnear=1
         if(indxx.EQ.ix2 .AND. indxy.EQ.iy1) ijnear=2
         if(indxx.EQ.ix1 .AND. indxy.EQ.iy2) ijnear=3
         if(indxx.EQ.ix2 .AND. indxy.EQ.iy2) ijnear=4
         if(ijnear.EQ.0) then
            write(iolst,*)'COMP:  Inconsistent cell results'
            write(iolst,*)'  Nearest Cell must be a Surrounding Cell'
c            stop
            iret=1
            estrng='Error in COMP -- see list file'
            return
         endif

c ---    Write the heading for the list file table
         write(iolst,6050)
6050     format(//1x,'THE FOLLOWING SOUNDINGS HAVE BEEN PROCESSED'//1x,
     1          6X,'YEAR',3X,'MONTH',3X,'DAY',3X,'JULIAN DAY',3X,
     2         'HOUR (GMT)',3X,'NO. LEVELS EXTRACTED'/)

         do i=1,nhrs
            do j=1,ny
               do k=1,nx
                  read(ioinp,65,end=99) iyr,imo,ida,ihr,ix,iy
65                format(i4,3i2,2i3)
c ---             Set profile index
                  ip=5
                  if(ix.EQ.ix1 .AND. iy.EQ.iy1) ip=1
                  if(ix.EQ.ix2 .AND. iy.EQ.iy1) ip=2
                  if(ix.EQ.ix1 .AND. iy.EQ.iy2) ip=3
                  if(ix.EQ.ix2 .AND. iy.EQ.iy2) ip=4
                  do l=1,nl
                      read(ioinp,70) ipres,iht,t(ip,l),idir,spd(ip,l)
70                    format(i4,1x,i5,1x,f5.1,1x,i3,1x,f4.1)
                      if(ipres.GT.iptop) istop=l+1
                      pres(ip,l)=FLOAT(ipres)
                      ht(ip,l)=FLOAT(iht)
                      dir(ip,l)=FLOAT(idir)
                  enddo
               enddo
            enddo

c ---       Screen for reporting times requested
            k24hr=ihr
            if(k24hr.EQ.0) k24hr=24
            if(mtimes.EQ.2) then
               ioutput=1
            elseif(mtimes.EQ.1 .AND. mod(ihr,12).EQ.0) then
               ioutput=1
            elseif(mtimes.EQ.3 .AND. hrmask(k24hr).EQ.1) then
               ioutput=1
            else
               ioutput=0
            endif
            if(ioutput.EQ.1) then
               istop=MIN(istop,nl)

c ---          Extract nearest profile or interpolate to station
c ---          (place in profile index 5)
               if(mspace.EQ.1) then
                  do lz=1,nl
                     pres(5,lz)=pres(ijnear,lz)
                     ht(5,lz)=ht(ijnear,lz)
                     t(5,lz)=t(ijnear,lz)
                     dir(5,lz)=dir(ijnear,lz)
                     spd(5,lz)=spd(ijnear,lz)
                  enddo
               elseif(mspace.EQ.2) then
                  do lz=1,nl
                     call INTERPH(pres(1,lz),wgts,pres(5,lz))
                     call INTERPH(ht(1,lz),wgts,ht(5,lz))
                     call INTERPH(t(1,lz),wgts,t(5,lz))
                     call INTERPHWIND(spd(1,lz),dir(1,lz),wgts,
     &                                spd(5,lz),dir(5,lz))
                  enddo
               endif

c ---          Search for minimum height (m MSL) of first level
               iht=NINT(ht(5,1))
               ielevm=MIN(ielevm,iht)
c ---          Update date/time for 'last' entry
               iyre=iyr
               imoe=imo
               idae=ida
               ihre=ihr
               call JULDAY(iolst,iyre,imoe,idae,ijule)
               if(iret.NE.0) then
                  klen=LEN_TRIM(estrng)
                  estrng='COMP:'//estrng(1:klen)
                  return
               endif
               if(LFIRST) then
c ---             Store starting time for output data
                  iyrs=iyre
                  ijuls=ijule
                  ihrs=ihre
                  lfirst=.FALSE.
               endif
c ---          Report to list file
               write(iolst,6060) iyre,imoe,idae,ijule,ihre,istop
6060           format(7X,I4,4X,I2,6X,I2,7X,I3,9X,I2,14X,I5)

c ---          Set the end-time
               ieyrn=iyr
               iejuln=ijule
               iehrn=ihr
               iesecn=0
c ---          For ending time, use midnight=24h on day n, rather than 0h on day n+1
               if (iehrn.eq.0) then
                  call incr(iolst,ieyrn,iejuln,iehrn,-1)
                  iehrn=iehrn+1
               endif
               call grday(iolst,ieyrn,iejuln,iemon,iedyn)
               if(iret.NE.0) then
                  klen=LEN_TRIM(estrng)
                  estrng='COMP:'//estrng(1:klen)
                  return
               endif
c ---          Enforce snapshot convention (begin=end time)
               ibyrn=ieyrn
               ibjuln=iejuln
               ibmon=iemon
               ibdyn=iedyn
               ibhrn=iehrn
               ibsecn=iesecn

c ---          Explicit beg/ending times with seconds (format 2.1)
               write(iotmp,6200) idsta,ibyrn,ibmon,ibdyn,ibhrn,
     &                           ibsecn,ieyrn,iemon,iedyn,iehrn,
     &                           iesecn,nl,istop
6200  FORMAT(3X,'9999',2X,i5,3x,2(4x,i4,i4,i3,i3,i5),i5,3x,i5)

               if(istop.EQ.nl) then
c ---             Error Message
                  write(iotmp,61)
                  write(iolst,61)
61    format(' ->->->Data at top of sounding is missing')
               endif
c ---          Write a comma-delimited record format
               istopm1=istop-1
               write(iotmp,6211) 
     1              (NINT(pres(5,m)),NINT(ht(5,m)),t(5,m),
     2               NINT(dir(5,m)),spd(5,m),ccomma,m=1,istopm1),
     3               NINT(pres(5,istop)),NINT(ht(5,istop)),t(5,istop),
     4               NINT(dir(5,istop)),spd(5,istop)
6211           format(4(3X,i4,'.0,',i4,'.,',f5.1,',',i3,',',
     &                f5.1,a1))
            endif
         enddo

99       close(ioinp)
         write(iolst,6090)iyr,imo,ida,ihr
6090     format(/20X,'EOF ON INPUT'/20X,'LAST PERIOD READ =  ',I4,3I3)

      enddo

c --- All M3D files processed, so move results from scratch file to
c --- output file with updated header information
      REWIND(iotmp)

c --- Create header comment strings to document extraction location
      kjust=LEN_TRIM(justname(nm3d))
      write(comment(1),'(a)')'Last M3D File Processed: '//
     &                       justname(nm3d)(1:kjust)
      if(mspace.EQ.1) then
         write(comment(2),'(a17,i4,a1,i4)')'Extraction Cell: ',
     &                                     indxx,ccomma,indxy
      elseif(mspace.EQ.2) then
         write(comment(2),'(a17,f8.3,a1,f8.3)')'Extraction Cell: ',
     &                                     xsi,ccomma,ysj
      endif

c --- Write header records
      call WRHD(iform,ioout,iyrs,ijuls,ihrs,iyre,ijule,ihre,
     1          pstop,jdat,L1,L2,L3,L4,
     2          ielevm,idsta,cname,clat,clon,ver,level,
     3          comment)
      if(iret.NE.0) then
         klen=LEN_TRIM(estrng)
         estrng='COMP:'//estrng(1:klen)
         return
      endif

c --- Grab data records from previous UP.DAT file
      if(LPREV) then
900      read(ioprev,'(a128)',end=990) aline
         write(ioout,'(a128)') aline
         goto 900
      endif

c --- Grab data records from scratch file
990   read(iotmp,'(a128)',end=999) aline
      write(ioout,'(a128)') aline
      goto 990

999   return
      end

c ---------------------------------------------------------------------
      subroutine interph(x,wgt,y)
c ---------------------------------------------------------------------
c
c --- UAMAKE   Version: 1.53      Level: 090225
c --- Zhong-Xiang Wu           
c
c --- PURPOSE:  Horizontal interpolation of scalars
c
c NOTE:  Array elements 1-4 are ordered as
c        3=UL     4=UR
c        1=LL     2=LR
c ---------------------------------------------------------------------

      dimension x(4),wgt(4)

      y=x(1)*wgt(1)*wgt(3)+x(2)*wgt(2)*wgt(3)+
     &  x(3)*wgt(1)*wgt(4)+x(4)*wgt(2)*wgt(4)

      return
      end
c ---------------------------------------------------------------------
      subroutine interphwind(wsintp,wdintp,wgt,wsfin,wdfin)
c ---------------------------------------------------------------------
c
c --- UAMAKE   Version: 1.53      Level: 090225
c --- Zhong-Xiang Wu           
c
c --- PURPOSE:  Horizontal interpolation of wind speed and wind
c               direction
c ---------------------------------------------------------------------

      dimension wsintp(4),wdintp(4),wgt(4)
      dimension u(4),v(4)
      data factor/57.2957795/

      do i=1,4
         fwd=wdintp(i)/factor
         fws=wsintp(i)
         u(i)=-fws*sin(fwd)
         v(i)=-fws*cos(fwd)
      enddo

      uext=u(1)*wgt(1)*wgt(3)+u(2)*wgt(2)*wgt(3)+
     &     u(3)*wgt(1)*wgt(4)+u(4)*wgt(2)*wgt(4)
      vext=v(1)*wgt(1)*wgt(3)+v(2)*wgt(2)*wgt(3)+
     &     v(3)*wgt(1)*wgt(4)+v(4)*wgt(2)*wgt(4)

      wsfin=sqrt(uext*uext+vext*vext)
      if(wsfin.lt.1.0E-5) then
         angle=0.
      else
         angle = 270.-atan2(vext,uext)*factor
      endif
      wdfin = amod(angle,360.)      

c      if (wdfin .eq. 0.) wdfin = 360.

      return
      end


c----------------------------------------------------------------------
	Subroutine getver(buff1,buff2,ivs3)
c----------------------------------------------------------------------
C
C --- UAMAKE     Version: 1.53    Level: 050208                  GETVER
C
C --- (subroutine taken from MM5COMB2   Version: 2.0   Level: 040116)
C     Zhong-Xiang Wu
C     1/16/2004
c
c --- UPDATES:
c --- (050208) to V 1.52 Level 090225 (DGS)
c           - /ERR/ processing
C
C --- Purpose: Get 3D.DAT format/version number
C 
C --- Input: BUFF1, BUFF2 - First two records
C --- Output: ivs3 - 3D.DAT format key number
c
C --- Called by: Main
C --- Calling: None
c----------------------------------------------------------------------

        include 'params.uas'
        include 'filnam.uas'
        include 'err.cal'

	character*132 buff1,buff2
	character*12 cset3d

	ivs3=-999
	read(buff2,'(a12)')cset3d
	if(cset3d.eq.'3D.DAT'.or.cset3d.eq.'MM53D.DAT')then
	   ivs3=1
	else
c ---      Step (2) check for new standard 3D.DAT file structure 
c ---      with dataset name, version and comments on first record
	   ncomm=-999
	   if(buff1(1:6).eq.'3D.DAT')then
	      read(buff2,*,err=1000)ncomm
	      if(ncomm.ge.1) then
		 ivs3=2
		 goto 2000
	      endif
 1000	      continue		! no comment lines, assume old MM5 format
	      write(iolst,*)'3D.DAT with comment line format'
	      write(iolst,*)'But no comment lines'
	      write(iolst,*)'Check/correct the 3D.DAT'
c             stop
              iret=1
              estrng='Error in GETVER -- see list file'
              return
	   else
c ---         Structure does not fit 3D.DAT conventions -- assume
c ---         file is in MM5.DAT format
	      ivs3=0
	   endif
	endif

 2000	write(iolst,*)'(File format - ivs3 = ',ivs3,', ncomm = ',ncomm,')'

	if(ivs3.lt.0 .or. ivs3.gt.2) then
	   write(iolst,*)'Illegal 3D.DAT format. Check data files'
c	   stop
           iret=1
           estrng='Error in GETVER -- see list file'
	endif
 
	return
	end

c----------------------------------------------------------------------
      subroutine wrhd(iform,io,ibyr,ibjul,ibhr,ieyr,iejul,iehr,
     1                pstop,jdat,LHT,LTEMP,LWD,LWS,
     2                ielevm,idsta,cname,clat,clon,cver,clev,
     3                comment)
c----------------------------------------------------------------------
c
c --- UAMAKE     Version: 1.53    Level: 090225                   WRHD
c ---            J. Scire, D. Strimaitis
c
c --- Write the header records for the UP.DAT file
c
c --- UPDATES:
c
c --- V 1.51 Level 080919  ====> V 1.52 Level 090225 (DGS)
c           - Add 2 new comment records passed as arguments
c           - /ERR/ processing
c
c --- V 1.5 Level 050208  ====> V 1.51 Level 080919 (DGS)
c           - Move the time zone record so that it appears just
c             before the time information record when station
c             location information is written
c
c --- Level 041122-2 (I. Lee)
c           - Write explicit beginning/ending times with seconds 
c             (not hour-ending convention). New UP.DAT format 2.1
c           - Write out time zone in header (time zone is O as
c             UTC/GMT time is used in UP.DAT) - Extra line -
c      
c --- Level 041122-1 (I. Lee)
c           - Modified READ62.FOR subroutine WRHD for UAMAKE.FOR 
c
c
c----------------------------------------------------------------------
c --- TAKEN FROM READ62     Version: 5.53     Level: 030402
c----------------------------------------------------------------------
c
c
c --- INPUTS:
c             IFORM - integer    - Data format flag (1=slash delimiter,
c                                  2=comma delimiter - more precision)
c              IBYR - integer    - Beginning year of data (4 digits)
c             IBJUL - integer    - Beginning Julian day number
c              IBHR - integer    - Beginning hour
c              IEYR - integer    - Ending year of data (4 digits)
c             IEJUL - integer    - Ending Julian day number
c              IEHR - integer    - Ending hour
c             PSTOP - real       - Top pressure level (mb) of data
c              JDAT - integer    - Original data format (1=TD-6201,
c                                  2=NCDC FSL)
c               LHT - logical    - Drop Sounding level if no height
c             LTEMP - logical    - Drop Sounding level if no temperature
c               LWD - logical    - Drop Sounding level if no direction
c               LWS - logical    - Drop Sounding level if no speed
c            IELEVM - integer    - station elevation (m MSL)
c             IDSTA - integer    - station identification code
c          CNAME - character*4   - Name of each station (----)
c           CLAT - character*16  - Latitude of each station (deg[N/S])
c           CLON - character*16  - Longitude of each station (deg[E/W])
c           CVER - character*12  - Version of processor
c           CLEV - character*12  - Level of processor
c     COMMENT(2) - character*132 - 2 Comment records
c
c --- OUTPUT:  none
c
c --- WRHD called by:  COMP
c --- WRHD calls:      none
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.uas'
      include 'err.cal'
c
      logical lht,ltemp,lwd,lws

      character*4 cname
      character*16 clat,clon
      character*12 cver,clev
      character*1 q

      character*4 xyunit
      character*8 datum, pmap
      character*12 daten
      character*16 dataset,dataver
      character*64 datamod
      character*80 comment1
      character*132 comment(2)

c --- Configure output variables
      data dataset/'UP.DAT'/, dataver/'2.1'/
      data datamod/'Hour Start and End Times with Seconds'/
      data ncomment/3/
      data comment1/'Produced by UAMAKE Version: '/

c --- Set single quote character
      data q/''''/

c --- Construct the version-level comment string
      j=29
      do i=1,12
         if(cver(i:i).NE.' ') then
            comment1(j:j)=cver(i:i)
            j=j+1
         endif
      enddo
      j=j+1
      comment1(j:j+7)=' Level: '
      j=j+8
      do i=1,12
         if(clev(i:i).NE.' ') then
            comment1(j:j)=clev(i:i)
            j=j+1
         endif
      enddo

c --- Set map projection information
      if(envpop.EQ.0) then
         pmap='NONE    '
      elseif(envpop.EQ.1) then
         pmap='LL      '
         datum='WGS-G   '
         daten='10-10-2002  '
         xyunit='DEG '
      endif


c --- Convert single hour-ending time to explicit beg/end times with seconds
c --- for output (041107)
      ibyrn=ibyr
      ibjuln=ibjul
      ibhrn=ibhr
      ibsecn=0
      call incr(io6,ibyrn,ibjuln,ibhrn,-1)
      ieyrn=ieyr
      iejuln=iejul
      iehrn=iehr      
      iesecn=0
c --- If ending time, use 24 to refer to midnight, not next day 0h
      if (iehrn.eq.0) then
         call incr(io6,ieyrn,iejuln,iehrn,-1)
         iehrn=iehrn+1
      endif


c --- Write header
      write(io,'(2a16,a64)') dataset,dataver,datamod
      write(io,'(i4)') ncomment
      write(io,'(a80)') comment1
      do i=1,ncomment-1
c         comment(i)=ADJUSTL(comment(i))
         last=LEN_TRIM(comment(i))
         write(io,'(a)') comment(i)(1:last)
      enddo
      write(io,'(a8)') pmap

      if(envpop.EQ.0) then
c ---    Header without location data
         write(io,'(a8)')'UTC+0000'
         WRITE (io,'(1X,8I5,F5.0,2I5)')IBYRN,IBJULN,IBHRN,IBSECN,
     &                        IEYRN,IEJULN,IEHRN,IESECN,PSTOP,JDAT,iform
         WRITE (io,'(1X,4(4X,L1))')LHT,LTEMP,LWD,LWS
      elseif(envpop.EQ.1) then
c ---    Header with location data
         write(io,'(a8,a12)') datum,daten
         write(io,'(a4)') xyunit
         write(io,'(a8)')'UTC+0000'
         WRITE (io,'(1X,8I5,F5.0,2I5)')IBYR,IBJULN,IBHRN,IBSECN,
     &                        IEYRN,IEJULN,IEHRN,IESECN,PSTOP,JDAT,iform
         WRITE (io,'(1X,4(4X,L1))')LHT,LTEMP,LWD,LWS
         write(io,20)idsta,q,cname,q,q,clat,q,q,clon,q,ielevm
      else
         write(iolst,*)'WRHD:  invalid parameter ENVPOP in PARAMS.UAS'
         write(iolst,*)'       Expected 0 or 1'
         write(iolst,*)'       Found ',envpop
c         stop
         iret=1
         estrng='Error in WRHD -- see list file'
      endif

20    format(i8,2x,a1,a4,a1,2x,2(a1,a16,a1,2x),i7)

      return
      end
